@@ -1,44 +1,34 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_graph.h"
 
 #include "tbb/task_scheduler_init.h"
-#include "tbb/spin_mutex.h"
-
-tbb::spin_mutex global_mutex;
+#include "tbb/spin_rw_mutex.h"
 
 #define N 100
 #define MAX_NODES 4
 
 //! Performs test on function nodes with limited concurrency and buffering
 /** Theses tests check:
-    1) that the number of executing copies never exceed the concurreny limit
+    1) that the number of executing copies never exceed the concurrency limit
     2) that the node never rejects
     3) that no items are lost
     and 4) all of this happens even if there are multiple predecessors and successors
@@ -73,21 +63,24 @@ void buffered_levels( size_t concurrency, Body body ) {
 
    // Set the execute_counter back to zero in the harness
    harness_graph_executor<InputType, OutputType>::execute_count = 0;
+   // Set the number of current executors to zero.
+   harness_graph_executor<InputType, OutputType>::current_executors = 0;
    // Set the max allowed executors to lc.  There is a check in the functor to make sure this is never exceeded.
    harness_graph_executor<InputType, OutputType>::max_executors = lc;
 
-   // Create the function_node with the appropriate concurreny level, and use default buffering
+   // Create the function_node with the appropriate concurrency level, and use default buffering
    tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, body );
    tbb::flow::function_node<InputType, InputType> pass_thru( g, tbb::flow::unlimited, pass_through<InputType>());
    
-   //Create a vector of identical exe_nodes and pass_thrus
-   // attach each pass_thru to its corresponding exe_node
+   // Create a vector of identical exe_nodes and pass_thrus
    std::vector< tbb::flow::function_node< InputType, OutputType > > exe_vec(2, exe_node);
    std::vector< tbb::flow::function_node< InputType, InputType > > pass_thru_vec(2, pass_thru);
+   // Attach each pass_thru to its corresponding exe_node
    for (size_t node_idx=0; node_idx<exe_vec.size(); ++node_idx) {
        tbb::flow::make_edge(pass_thru_vec[node_idx], exe_vec[node_idx]);
    }
 
+   // TODO: why the test is executed serially for the node pairs, not concurrently?
    for (size_t node_idx=0; node_idx<exe_vec.size(); ++node_idx) {
    // For num_receivers = 1 to MAX_NODES
    for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
@@ -100,12 +93,11 @@ void buffered_levels( size_t concurrency, Body body ) {
         // Do the test with varying numbers of senders
         harness_counting_sender<InputType> *senders = NULL;
         for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
-            // Create num_senders senders, set there message limit each to N, and connect them to the exe_vec[node_idx]
+            // Create num_senders senders, set there message limit each to N, and connect them to pass_thru_vec[node_idx]
             senders = new harness_counting_sender<InputType>[num_senders];
             for (size_t s = 0; s < num_senders; ++s ) {
                senders[s].my_limit = N;
-               // tbb::flow::make_edge( senders[s], exe_vec[node_idx] );
-               tbb::flow::make_edge( senders[s], pass_thru_vec[node_idx] );
+               senders[s].register_successor(pass_thru_vec[node_idx] );
             }
 
             // Initialize the receivers so they know how many senders and messages to check for
@@ -117,7 +109,7 @@ void buffered_levels( size_t concurrency, Body body ) {
             NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
             g.wait_for_all();
 
-            // cofirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times 
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
                 ASSERT( n == N, NULL ); 
@@ -139,9 +131,9 @@ void buffered_levels( size_t concurrency, Body body ) {
             receivers[r].validate();
         }
         delete [] receivers;
-    }
-    } 
-    }
+    } // for num_receivers
+    } // for node_idx
+    } // for concurrency level lc
 }
 
 const size_t Offset = 123;
@@ -152,6 +144,7 @@ struct inc_functor {
     tbb::atomic<size_t> local_execute_count;
     inc_functor( ) { local_execute_count = 0; }
     inc_functor( const inc_functor &f ) { local_execute_count = f.local_execute_count; }
+    void operator=( const inc_functor &f ) { local_execute_count = f.local_execute_count; }
 
     int operator()( int i ) {
        ++global_execute_count;
@@ -222,17 +215,22 @@ void buffered_levels_with_copy( size_t concurrency ) {
         size_t global_count = global_execute_count;
         size_t inc_count = body_copy.local_execute_count;
         ASSERT( global_count == expected_count && global_count == inc_count, NULL ); 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        g.reset(tbb::flow::rf_reset_bodies);
+        body_copy = tbb::flow::copy_body<inc_functor>( exe_node );
+        inc_count = body_copy.local_execute_count;
+        ASSERT( Offset == inc_count, "reset(rf_reset_bodies) did not reset functor" ); 
+#endif
     }
 }
 
 template< typename InputType, typename OutputType >
 void run_buffered_levels( int c ) {
-    harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::max_executors = c;
     #if __TBB_LAMBDAS_PRESENT
-    buffered_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func(i); } );
+    buffered_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::func(i); } );
     #endif
-    buffered_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func );
-    buffered_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::functor() );
+    buffered_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::func );
+    buffered_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType>::functor() );
     buffered_levels_with_copy<InputType,OutputType>( c );
 }
 
@@ -251,24 +249,46 @@ void concurrency_levels( size_t concurrency, Body body ) {
 
    for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
    tbb::flow::graph g;
-   harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::execute_count = 0;
 
-   tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > exe_node( g, lc, body );
+   // Set the execute_counter back to zero in the harness
+   harness_graph_executor<InputType, OutputType>::execute_count = 0;
+   // Set the number of current executors to zero.
+   harness_graph_executor<InputType, OutputType>::current_executors = 0;
+   // Set the max allowed executors to lc. There is a check in the functor to make sure this is never exceeded.
+   harness_graph_executor<InputType, OutputType>::max_executors = lc;
+
+   typedef tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > fnode_type;
+   fnode_type exe_node( g, lc, body );
 
    for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
 
         harness_counting_receiver<OutputType> *receivers = new harness_counting_receiver<OutputType>[num_receivers];
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT(exe_node.successor_count() == 0, NULL);
+        ASSERT(exe_node.predecessor_count() == 0, NULL);
+#endif
+
         for (size_t r = 0; r < num_receivers; ++r ) {
             tbb::flow::make_edge( exe_node, receivers[r] );
         }
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT(exe_node.successor_count() == num_receivers, NULL);
+        typename fnode_type::successor_vector_type my_succs;
+        exe_node.copy_successors(my_succs);
+        ASSERT(my_succs.size() == num_receivers, NULL);
+        typename fnode_type::predecessor_vector_type my_preds;
+        exe_node.copy_predecessors(my_preds);
+        ASSERT(my_preds.size() == 0, NULL);
+#endif
 
         harness_counting_sender<InputType> *senders = NULL;
 
         for (size_t num_senders = 1; num_senders <= MAX_NODES; ++num_senders ) {
+            senders = new harness_counting_sender<InputType>[num_senders];
             {
-                // lock m to prevent exe_node from finishing
-                tbb::spin_mutex::scoped_lock l( harness_graph_executor< InputType, OutputType, tbb::spin_mutex >::mutex );
+                // Exclusively lock m to prevent exe_node from finishing
+                tbb::spin_rw_mutex::scoped_lock l( harness_graph_executor<InputType, OutputType>::template mutex_holder<tbb::spin_rw_mutex>::mutex );
 
                 // put to lc level, it will accept and then block at m
                 for ( size_t c = 0 ; c < lc ; ++c ) {
@@ -277,7 +297,6 @@ void concurrency_levels( size_t concurrency, Body body ) {
                 // it only accepts to lc level
                 ASSERT( exe_node.try_put( InputType() ) == false, NULL );
 
-                senders = new harness_counting_sender<InputType>[num_senders];
                 for (size_t s = 0; s < num_senders; ++s ) {
                    // register a sender
                    senders[s].my_limit = N;
@@ -288,13 +307,13 @@ void concurrency_levels( size_t concurrency, Body body ) {
             // wait for graph to settle down
             g.wait_for_all();
 
-            // cofirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times 
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
                 ASSERT( n == N, NULL ); 
                 ASSERT( senders[s].my_receiver == &exe_node, NULL );
             }
-            // cofirm that each receivers got N * num_senders + the initial lc puts
+            // confirm that each receivers got N * num_senders + the initial lc puts
             for (size_t r = 0; r < num_receivers; ++r ) {
                 size_t n = receivers[r].my_count;
                 ASSERT( n == num_senders*N+lc, NULL );
@@ -318,12 +337,11 @@ void concurrency_levels( size_t concurrency, Body body ) {
 
 template< typename InputType, typename OutputType >
 void run_concurrency_levels( int c ) {
-    harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::max_executors = c;
     #if __TBB_LAMBDAS_PRESENT
-    concurrency_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func(i); } );
+    concurrency_levels<InputType,OutputType>( c, []( InputType i ) -> OutputType { return harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex>(i); } );
     #endif
-    concurrency_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::func );
-    concurrency_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType, tbb::spin_mutex>::functor() );
+    concurrency_levels<InputType,OutputType>( c, &harness_graph_executor<InputType, OutputType>::template tfunc<tbb::spin_rw_mutex> );
+    concurrency_levels<InputType,OutputType>( c, typename harness_graph_executor<InputType, OutputType>::template tfunctor<tbb::spin_rw_mutex>() );
 }
 
 
@@ -398,7 +416,7 @@ void run_unlimited_concurrency() {
     unlimited_concurrency<InputType,OutputType>( typename harness_graph_executor<InputType, OutputType>::functor() );
 }
 
-struct continue_msg_to_int : private NoAssign {
+struct continue_msg_to_int {
     int my_int;
     continue_msg_to_int(int x) : my_int(x) {}
     int operator()(tbb::flow::continue_msg) { return my_int; }
@@ -435,8 +453,114 @@ void test_concurrency(int num_threads) {
     test_function_node_with_continue_msg_as_input();
 }
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+struct add_to_counter {
+    int* counter;
+    add_to_counter(int& var):counter(&var){}
+    int operator()(int i){*counter+=1; return i + 1;}
+};
+
+template<tbb::flow::graph_buffer_policy FTYPE>
+void test_extract() {
+    int my_count = 0;
+    int cm;
+    tbb::flow::graph g;
+    tbb::flow::broadcast_node<int> b0(g);
+    tbb::flow::broadcast_node<int> b1(g);
+    tbb::flow::function_node<int, int, FTYPE> f0(g, tbb::flow::unlimited, add_to_counter(my_count));
+    tbb::flow::queue_node<int> q0(g);
+
+    tbb::flow::make_edge(b0, f0);
+    tbb::flow::make_edge(b1, f0);
+    tbb::flow::make_edge(f0, q0);
+    for( int i = 0; i < 2; ++i ) {
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 1, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 2 && f0.successor_count() == 1, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 1 && q0.successor_count() == 0, "q0 has incorrect counts");
+    
+        /* b0         */
+        /*   \        */
+        /*    f0 - q0 */
+        /*   /        */
+        /* b1         */
+    
+        b0.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 1, "function_node didn't fire");
+        ASSERT(q0.try_get(cm), "function_node didn't forward");
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 2, "function_node didn't fire");
+        ASSERT(q0.try_get(cm), "function_node didn't forward");
+    
+        b0.extract();
+    
+        /* b0         */
+        /*            */
+        /*    f0 - q0 */
+        /*   /        */
+        /* b1         */
+    
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 1, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 1 && q0.successor_count() == 0, "q0 has incorrect counts");
+        b0.try_put(1);
+        b0.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 2, "b0 messages being forwarded to function_node even though it is disconnected");
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
+        ASSERT(q0.try_get(cm), "function_node didn't forward second time");
+    
+        f0.extract();
+    
+        /* b0         */
+        /*            */
+        /*    f0   q0 */
+        /*            */
+        /* b1         */
+    
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 0 && f0.successor_count() == 0, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 0 && q0.successor_count() == 0, "q0 has incorrect counts");
+        b0.try_put(1);
+        b0.try_put(1);
+        b1.try_put(1);
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
+        ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
+        make_edge(b0, f0);
+    
+        /* b0         */
+        /*   \        */
+        /*    f0   q0 */
+        /*            */
+        /* b1         */
+    
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 1, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 0, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 0 && q0.successor_count() == 0, "q0 has incorrect counts");
+    
+        b0.try_put(int());
+        g.wait_for_all();
+    
+        ASSERT(my_count == 4, "function_node didn't fire though it has only one predecessor");
+        ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
+    
+        tbb::flow::make_edge(b1, f0);
+        tbb::flow::make_edge(f0, q0);
+        my_count = 0;
+    }
+}
+#endif
+
 int TestMain() { 
-    current_executors = 0;
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -444,6 +568,11 @@ int TestMain() {
     for( int p=MinThread; p<=MaxThread; ++p ) {
        test_concurrency(p);
    }
+
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    test_extract<tbb::flow::rejecting>();
+    test_extract<tbb::flow::queueing>();
+#endif
    return Harness::Done;
 }
 
