@@ -1,32 +1,30 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
-#if !(_WIN32||_WIN64)
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+    #pragma warning (push)
+    #pragma warning (disable: 4530)
+#endif
+
+#if !(_WIN32||_WIN64) || (__MINGW64__||__MINGW32__)
 
 #include "harness.h"
 
@@ -45,27 +43,21 @@ int TestMain () {
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
-
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
-    #pragma warning (push)
-    #pragma warning (disable: 4530)
-#endif
-
 #include <vector>
 #include <string>
 #include <utility>
 #include <typeinfo>
-
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    #pragma warning (pop)
-#endif
+#include <stdexcept>
 
 #ifdef HARNESS_USE_RUNTIME_LOADER
     #undef HARNESS_USE_RUNTIME_LOADER    // We do not want harness to preload tbb.
 #endif
 #include "harness.h"
 
+#if !TBB_USE_EXCEPTIONS && _MSC_VER
+    #pragma warning (pop)
+#endif
+
 static int errors = 0;
 
 #define CHECK( cond ) {                                                  \
@@ -204,7 +196,7 @@ int TestMain() {
                     // Check what?
                 }
 
-                // There was a problem on Linux, and still a problem on Mac OS X.
+                // There was a problem on Linux* OS, and still a problem on OS X*.
                 SAY( "Throw an exception." );
                 // Iterate thru all the ids first.
                 for ( int id = 1; id < tbb::internal::eid_max; ++ id ) {
@@ -244,6 +236,7 @@ int TestMain() {
                 CHECK_EXCEPTION( eid_improper_lock,               tbb::improper_lock               );
                 CHECK_EXCEPTION( eid_possible_deadlock,           std::runtime_error               );
                 CHECK_EXCEPTION( eid_reservation_length_error,    std::length_error                );
+                CHECK_EXCEPTION( eid_user_abort,                  tbb::user_abort                  );
                 #undef CHECK_EXCEPTION
                 {
                     bool ex_caught = false;
